<?php

if ( !in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
    return;
}

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

const OPT_MECOM_AIRWALLEX_VERSION = '1.0.4';
const Opt_Mecom_Airwallex_Proxies = 'Opt_Mecom_Airwallex_Proxies';
const Opt_Mecom_Airwallex_Activated_Proxy = 'Opt_Mecom_Airwallex_Activated_Proxy';
const OPT_MECOM_AIRWALLEX_ROTATION_METHOD =  'OPT_MECOM_AIRWALLEX_ROTATION_METHOD';
const OPT_MECOM_AIRWALLEX_UNUSED_PROXIES = 'OPT_MECOM_AIRWALLEX_UNUSED_PROXIES';
const OPT_MECOM_AIRWALLEX_CURRENT_ROTATION_VALUE = 'OPT_MECOM_AIRWALLEX_CURRENT_ROTATION_VALUE';
const MetaKey_Airwallex_Proxy_Url = '_mecom_airwallex_proxy_url';
const OPT_MECOM_AIRWALLEX_LAST_TIME_RESET_PAID_AMOUNT = 'OPT_MECOM_AIRWALLEX_LAST_TIME_RESET_PAID_AMOUNT';

const OPT_MECOM_AIRWALLEX_INTENT_CAPTURE = 'OPT_MECOM_AIRWALLEX_INTENT_CAPTURE';
const OPT_MECOM_AIRWALLEX_INTENT_AUTHORIZE = 'OPT_MECOM_AIRWALLEX_INTENT_AUTHORIZE';

const METAKEY_MECOM_AIRWALLEX_INTENT_AUTHORIZED = '_METAKEY_MECOM_AIRWALLEX_INTENT_CAPTURED';
const METAKEY_AIRWALLEX_PROXY_URL          = '_mecom_airwallex_proxy_url';
const METAKEY_AIRWALLEX_PROXY_URL_ERROR = '_METAKEY_AIRWALLEX_PROXY_URL_ERROR';
const MECOM_AIRWALLEX_BY_TIME                       = "by_time";
const MECOM_AIRWALLEX_BY_AMOUNT                     = "by_amount";

const METAKEY_CS_AIRWALLEX_FEE      = '_cs_airwallex_fee';
const METAKEY_CS_AIRWALLEX_PAYOUT   = '_cs_airwallex_payout';
const METAKEY_CS_AIRWALLEX_CURRENCY = '_cs_airwallex_currency';

// true: order currency
// false: airwallex currency
const MECOM_AIRWALLEX_FEE_DISPLAY_ORDER_CURRENCY = true;

function resetPaidAmountIfNeedAirwallex() {
    $lastTimeReset = get_option(OPT_MECOM_AIRWALLEX_LAST_TIME_RESET_PAID_AMOUNT, null);
    $proxies = get_option(Opt_Mecom_Airwallex_Proxies, []);
    if (empty($proxies)) {
        return [];
    }
    // Reset
    if (empty($lastTimeReset) || date('Y-m-d') > $lastTimeReset) {
        return resetPaidAmountAirwallex($proxies);
    }
    return $proxies;
}

function findActivatedProxyDataByIdAirwallex($proxies, $activatedProxyId) {
    foreach ($proxies as $proxy) {
        if ($proxy['id'] === $activatedProxyId) {
            return $proxy;
        }
    }
    return null;
}

function getNextProxyAmountRotationAirwallex($orderTotal) {
    $proxies = get_option(Opt_Mecom_Airwallex_Proxies, []);
    if (empty($proxies)) {
        return null;
    }
    $activatedProxy = get_option( Opt_Mecom_Airwallex_Activated_Proxy, null );
    if (empty($activatedProxy)) {
        airwallexLog("Activated proxy not found! Use the first proxy of rotation list");
        $activatedProxy = $proxies[0];
    }
    $activatedProxy = findActivatedProxyDataByIdAirwallex($proxies, $activatedProxy['id']);

    // Need to rotate proxy
    $isCurrentProxyMatched = false;
    foreach ($proxies as $proxy) {
        if($activatedProxy['id'] == $proxy['id'] && !$isCurrentProxyMatched) {
            $isCurrentProxyMatched = true;
            continue;
        }
        if($isCurrentProxyMatched && doubleval( $proxy['paid_amount'] ) + doubleval( $orderTotal ) < doubleval($proxy['amount'])) {
            return $proxy;
        }
    }
    foreach ($proxies as $proxy) {
        if(doubleval( $proxy['paid_amount'] ) + doubleval( $orderTotal ) < doubleval($proxy['amount'])) {
            return $proxy;
        }
    }
    return null;
}

function performProxyAmountRotationAirwallex($orderTotal) {
    $proxies = resetPaidAmountIfNeedAirwallex();
    if (empty($proxies)) {
        return null;
    }
    $activatedProxy = get_option( Opt_Mecom_Airwallex_Activated_Proxy, null );
    if (empty($activatedProxy)) {
        airwallexLog("Activated proxy not found! Use the first proxy of rotation list");
        $activatedProxy = $proxies[0];
        update_option(Opt_Mecom_Airwallex_Activated_Proxy, $activatedProxy, true);
        logAirwallexRotation(MECOM_AIRWALLEX_BY_AMOUNT, $activatedProxy, "Auto");
    }
    $activatedProxy = findActivatedProxyDataByIdAirwallex($proxies, $activatedProxy['id']);

    // Need to rotate proxy
    $isCurrentProxyMatched = false;
    foreach ($proxies as $proxy) {
        if($activatedProxy['id'] == $proxy['id'] && !$isCurrentProxyMatched) {
            $isCurrentProxyMatched = true;
            continue;
        }
        if($isCurrentProxyMatched && doubleval( $proxy['paid_amount'] ) + doubleval( $orderTotal ) < doubleval($proxy['amount'])) {
            $activatedProxy = $proxy;
            update_option(Opt_Mecom_Airwallex_Activated_Proxy, $activatedProxy, true);
            logAirwallexRotation(MECOM_AIRWALLEX_BY_AMOUNT, $activatedProxy, "Auto");
            return $activatedProxy;
        }
    }
    foreach ($proxies as $proxy) {
        if(doubleval( $proxy['paid_amount'] ) + doubleval( $orderTotal ) < doubleval($proxy['amount'])) {
            $activatedProxy = $proxy;
            update_option(Opt_Mecom_Airwallex_Activated_Proxy, $activatedProxy, true);
            logAirwallexRotation(MECOM_AIRWALLEX_BY_AMOUNT, $activatedProxy, "Auto");
            return $activatedProxy;
        }
    }
    return null;
}

function resetPaidAmountAirwallex($proxies = null)
{
    if (empty($proxies)) {
        $proxies = get_option( Opt_Mecom_Airwallex_Proxies, [] );
    }
    if (empty($proxies)) return [];
    // Reset
    $newProxies = array_map(function ($proxy) {
        $proxy['paid_amount'] = 0;
        return $proxy;
    }, $proxies);
    $unusedProxies = get_option(OPT_MECOM_AIRWALLEX_UNUSED_PROXIES, []);
    if (empty($unusedProxies)) {
        $newUnusedProxies = [];
    } else {
        $newUnusedProxies = array_map(function ($unusedProxy) {
            $unusedProxy['paid_amount'] = 0;
            return $unusedProxy;
        }, $unusedProxies);
    }
    update_option(Opt_Mecom_Airwallex_Proxies, $newProxies, true);
    update_option(OPT_MECOM_AIRWALLEX_UNUSED_PROXIES, $newUnusedProxies, true);
    update_option(OPT_MECOM_AIRWALLEX_LAST_TIME_RESET_PAID_AMOUNT, date('Y-m-d'), true);
    return $newProxies;
}

function logAirwallexRotation($rotationMethod, $proxy, $type)
{
    $mecom_log_dir = wp_get_upload_dir()["basedir"] . '/mecom';
    if (!is_dir($mecom_log_dir)) {
        mkdir($mecom_log_dir, 0777, true);
    }
    $logFilePath = $mecom_log_dir . '/'.getAirwallexLogFileName($rotationMethod);
    $currentDateTime             = date('Y-m-d H:i:s');
    $rotationValue               = $rotationMethod === MECOM_AIRWALLEX_BY_TIME ? $proxy['timestamp'] : ( $proxy['paid_amount'] . '/' . $proxy['amount']);
    $content                     = "{$currentDateTime} - {$proxy['url']} , {$rotationValue} - {$type}\n";

    $fp = fopen($logFilePath, 'a');
    fwrite($fp, $content);
    fclose($fp);
}

function getAirwallexLogFileName($rotationMethod) {
    if ( $rotationMethod === MECOM_AIRWALLEX_BY_TIME) {
        return 'mecom_airwallex_rotation_time_log.txt';
    } else if ( $rotationMethod === MECOM_AIRWALLEX_BY_AMOUNT) {
        return 'mecom_airwallex_rotation_amount_log.txt';
    }
}

function loadAirwallexLogs() {
    $mecom_log_dir = wp_get_upload_dir()["basedir"].'/mecom';
    $logFilePath = $mecom_log_dir.'/airwallex_rotation_log.txt';
    if (file_exists($logFilePath)) {
        $array = explode("\n", file_get_contents($logFilePath));   
    }else{
        $array = array();
    }
    return $array;
}

function isEnabledAmountRotationAirwallex() {
    return MECOM_AIRWALLEX_BY_AMOUNT === get_option(OPT_MECOM_AIRWALLEX_ROTATION_METHOD, MECOM_AIRWALLEX_BY_TIME);
}

function updateRotationAmountAirwallex($processedProxyId, $orderTotal) {
    $proxies = get_option(Opt_Mecom_Airwallex_Proxies, []);
    foreach ($proxies as $key => $proxy) {
        if ($proxy['id'] === $processedProxyId) {
            $proxies[$key]['paid_amount'] = doubleval($proxy['paid_amount']) + doubleval( $orderTotal );
            break;
        }
    }
    return update_option(Opt_Mecom_Airwallex_Proxies, $proxies, true);
}

function hasPayableProxyAirwallex($cartTotal) {
    $proxies = resetPaidAmountIfNeedAirwallex();
    if (empty($proxies)) {
        return false;
    }

    foreach ($proxies as $proxy) {
        if(doubleval( $proxy['paid_amount'] ) + doubleval( $cartTotal ) < doubleval($proxy['amount'])) {
            return true;
        }
    }
    return false;
}

function airwallexLog($error, $message = '')
{
    $logger = wc_get_logger();
    if (is_array($error) || is_object($error)) {
        $error = print_r($error, true);
    }
    $error = '\n--------------------- ' . $message . ' ---------------------\n'
        . $error .
        '\n------------------------------------------\n';
    if (empty($logger)) {
        error_log($error);
    } else {
        $logger->debug($error, ['source' => 'cardshield-gateway-airwallex']);
    }
}
